==> muppeteer-1.0.4-beta.tgz: package/README.md <==
# Muppeteer
[![Build Status](https://travis-ci.org/HuddleEng/Muppeteer.svg?branch=master)](https://travis-ci.org/HuddleEng/Muppeteer)

Muppeteer is a visual regression testing framework for running UI tests in Chrome. It's composed of a number of modules:

- [Mocha](https://mochajs.org/) - a test runner framework
- [Chai](http://chaijs.com/) - an assertion library
- [Puppeteer](https://github.com/GoogleChrome/puppeteer) - a library for interacting with Chrome and web pages
- [Pixelmatch](https://github.com/mapbox/pixelmatch) - a pixel-level image comparison library

In addition, it provides the following core features:
- **Visual Regression Testing** - a screenshot-based image comparison module that hooks onto the assertion API. Read on for more discussion on this.
- **Test Interface** - a modification of Mocha's BDD interface with built-in browser setup steps and other user configurable hooks
- **Test Launcher** - a CLI and configuration function for launching test suites
- **Page API** - the API for Muppeteer's page interactions

Muppeteer's main goal is to abstract the, often, tedious boilerplate setup code needed to write tests with Puppeteer, and provide a convenient API for testing UI functionality. It was inspired by the [PhantomCSS](https://github.com/HuddleEng/PhantomCSS) and [CasperJS](http://casperjs.org/) libraries.

**This framework is in beta pre-release currently. It is subject to some breaking changes until the API is finalised. There is also little test coverage as of yet, but it is in the works. Use with caution.**

- ## [Configuration](#configuration-1)
    - ### [CLI](#cli-1)
    - ### [Configuration function](#configuration-function-1)
- ## [API](https://github.com/HuddleEng/Muppeteer/blob/master/API.md)
- ## [Example test case](#example-test-case-1)
    - ### [Passing test output](#passing-test-output-1)
    - ### [Failing test output](#failing-test-output-1)
    - ### [Understanding visual failures](#understanding-visual-failures-1)

## Configuration
You can configure Muppeteer via the CLI or a configuration function

### CLI
The CLI script can be referenced at
 [`lib/test-launcher-cli`](https://github.com/HuddleEng/Muppeteer/blob/local-test-server/lib/test-launcher-cli.js).
 
 It is run like `node <<path-to-muppeteer>>/lib/test-launcher-cli <<args>>`
 
 #### Example
```javascript
 "scripts": {
    "test": "node node_modules/muppeteer/lib/test-launcher-cli --t tests --f test.js --r tests/report"
  }
```

See [Options](#options)
  
## Configuration function
The configuration can be referenced at 
 [`src/test-launcher`](https://github.com/HuddleEng/Muppeteer/blob/local-test-server/src/test-launcher.js).
 
 ### Example
```javascript
const ConfigureLauncher = require('../src/test-launcher');
const path = require('path');
const testsPath = path.join(__dirname, 'tests');

ConfigureLauncher({
        testDir: testsPath,
        testFilter: 'test.js',
        reportDir: `${testsPath}/report`,
        visualThreshold: 0.05,
        useDocker: true,
        onFinish: () => {
            // do something after the tests have complete
        }
    }
).launch();
```
## Options
**Note:** Only options with `--` can be run with the proceeding flag in the CLI interface.

- `testDir (--t)`: The directory for Mocha to look for the test files
- `testFilter (--f)`: Allows you to pass in some text to filter the test file name, it's just a substring match, nothing fancy
- `shouldRebaseVisuals (--b)`: A flag to tell the visual regression engine to replace the existing baseline visuals
- `reportDir (--r)`: The directory for the Mocha reporter to dump the report files
- `componentTestUrlFactory`: A function that returns the url for the component test to run
- `componentTestVisualPathFactory`: A function that returns the path for visual tests to run in
- `visualThreshold (--v)`: A value between 0 and 1 to present the threshold at which a visual test may pass or fail
- `onFinish`: A function that can be used to do some extra work after Muppeteer is teared down
- `useDocker (--d)`: The option for telling Muppeteer to run Chrome in Docker to better deal with environmental inconsistencies (default)
- `headless (--h)`: Determines whether Chrome will be launched in a headless mode (without GUI) or with a head  (not applicable with `useDocker`)
- `disableSandbox (--s)`: Used to disable the sandbox checks if not using [SUID sandbox](https://chromium.googlesource.com/chromium/src/+/master/docs/linux_suid_sandbox_development.md) (not applicable with `useDocker`)
- `executablePath (--e)`: The option to set the version of Chrome to use duning the tests. By default, it uses the bundled version (not applicable with `useDocker`)

## Example test case

```javascript
const container = '.todoapp';
const input = 'header input';
const listItem = '.todo-list li';
const firstItem = listItem + ':nth-of-type(1)';
const firstItemToggle = firstItem + ' .toggle';
const firstItemRemoveButton = firstItem + ' button';
const secondItem = listItem + ':nth-of-type(2)';
const todoCount = '.todo-count';

describeComponent({name: 'todomvc', url: 'http://somehost:3000'}, () => {
    describe('Add a todo item', async () => {
        it('typing text and hitting enter key adds new item', async () => {
            await page.waitForSelector(input);
            await page.type(input, 'My first item');
            await page.keyboard.press('Enter');
            await page.waitForSelector(firstItem);
            assert.equal(await page.getText(firstItem), 'My first item');
            await assert.visual(container);
        });
        it('clicking checkbox marks item as complete', async () => {
            await page.waitForSelector(firstItemToggle);
            await page.click(firstItemToggle);

            // something to break the tests
            // await page.addStyleTag({ content: '.header { padding-top: 50px; }'});

            await page.waitForNthSelectorAttributeValue(listItem, 1, 'class', 'completed');
            await assert.visual(container);
        });
        it('typing more text and hitting enter adds a second item', async () => {
            await page.type(input, 'My second item');
            await page.keyboard.press('Enter');
            await page.waitForSelector(secondItem);
            assert.equal(await page.getText(secondItem), 'My second item');
            await assert.visual(container);
        });
        it('hovering over first item shows x button', async () => {
            await page.hover(firstItem);
            await assert.visual(container);
        });
        it('clicking on first item x button removes it from the list', async () => {
            await page.click(firstItemRemoveButton);
            await page.waitForElementCount(listItem, 1);
            assert.equal(await page.getText(todoCount), '1 item left');
            await assert.visual(container);
        });
    });
});

```

## Docker and test fixtures
Muppeteer uses Docker by default to run tests. This helps to avoid environmental differences that could affect the 
rendering of content on the page. You can opt out by configuring the `useDocker (--d)` option accordingly.

If you are hosting your test fixtures on a local web server, you'd typically set the URL in the test to
something like http://localhost:3000. When using Docker, the `localhost` will refer to the container,
not the host machine. The simplest solution would be to reference the local IP in the test instead. For example,
http://192.168.0.4:3000.

However, this breaks down when you are running on a device you don't know how to address, e.g. a cloud CI agent.
To solve this problem, you can use the `componentTestUrlFactory` function in launch configuration to generate the URL. 
You can lookup the IP address of the current host and pass that through. This is used to run the example tests in this repo. 
See [run-tests](https://github.com/HuddleEng/Muppeteer/blob/master/example/run-tests.js) for an example.

```javascript
...
componentTestUrlFactory: () => `http://${IP}:${PORT}`
...
```

### Passing test output
![Passing tests](https://i.imgur.com/EOA3rJ6.png "Passing tests")

### Failing test output
![Failing tests tests](https://i.imgur.com/rPY6Bjq.png "Failing tests")

### Understanding visual failures

#### Baseline image ####
This is the visual that is versioned in your app repo. It is the source of truth.

![Baseline image](https://i.imgur.com/8dlSqyT.png "Baseline")

#### Current image ####
This is the screenshot taken during the test. In this example, we can see that some padding has 
pushed the text input field down.

![Current image](https://i.imgur.com/DVV3jvZ.png "Current")

#### Difference image ####
This is an image showing where the differences are. Each difference is layered on top of one another. Here we can see 
that the "What needs to be done?" placeholder has moved down, and so it's sitting on top of where "My first item" 
previously was. 

![Difference image](https://i.imgur.com/C1wpxc5.png "Difference")

==> muppeteer-1.1.1-beta.tgz: package/README.md <==
# Muppeteer
[![Build Status](https://travis-ci.org/HuddleEng/Muppeteer.svg?branch=master)](https://travis-ci.org/HuddleEng/Muppeteer)

Muppeteer is a visual regression testing framework for running UI tests in Chrome. It's composed of a number of modules:

- [Mocha](https://mochajs.org/) - a test runner framework
- [Chai](http://chaijs.com/) - an assertion library
- [Puppeteer](https://github.com/GoogleChrome/puppeteer) - a library for interacting with Chrome and web pages
- [Puppeteer Extensions](https://github.com/HuddleEng/puppeteer-extensions) - an extension library for Puppeteer with convenience functions
- [Pixelmatch](https://github.com/mapbox/pixelmatch) - a pixel-level image comparison library

In addition, it provides the following core features:
- **Visual Regression Testing** - a screenshot-based image comparison module that hooks onto the assertion API. Read on for more discussion on this.
- **Test Interface** - a modification of Mocha's BDD interface with built-in browser setup steps and other user configurable hooks
- **Test Launcher** - a CLI and configuration function for launching test suites

Muppeteer's main goal is to abstract the, often, tedious boilerplate setup code needed to write tests with Puppeteer, and provide a convenient API for testing UI functionality. It was inspired by the [PhantomCSS](https://github.com/HuddleEng/PhantomCSS) and [CasperJS](http://casperjs.org/) libraries.

**This framework is in beta pre-release currently. It is subject to some breaking changes until the API is finalised. There is also little test coverage as of yet, but it is in the works. Use with caution.**

- ## [Configuration](#configuration-1)
    - ### [CLI](#cli-1)
    - ### [Configuration function](#configuration-function-1)
- ## [Puppeteer API](https://github.com/GoogleChrome/puppeteer/blob/master/docs/api.md)
- ## [Puppeteer Extensions](#puppeteer-extensions-1)
- ## [Example test case](#example-test-case-1)
    - ### [Passing test output](#passing-test-output-1)
    - ### [Failing test output](#failing-test-output-1)
    - ### [Understanding visual failures](#understanding-visual-failures-1)

## Configuration
You can configure Muppeteer via the CLI or a configuration function

### CLI
The CLI script can be referenced at
 [`lib/test-launcher-cli`](https://github.com/HuddleEng/Muppeteer/blob/local-test-server/lib/test-launcher-cli.js).
 
 It is run like `node <<path-to-muppeteer>>/lib/test-launcher-cli <<args>>`
 
 #### Example
```javascript
 "scripts": {
    "test": "node node_modules/muppeteer/lib/test-launcher-cli --t tests --f test.js --r tests/report"
  }
```

See [Options](#options)
  
## Configuration function
The configuration can be referenced at 
 [`src/test-launcher`](https://github.com/HuddleEng/Muppeteer/blob/local-test-server/src/test-launcher.js).
 
 ### Example
```javascript
const ConfigureLauncher = require('../src/test-launcher');
const path = require('path');
const testsPath = path.join(__dirname, 'tests');

ConfigureLauncher({
        testDir: testsPath,
        testFilter: 'test.js',
        reportDir: `${testsPath}/report`,
        visualThreshold: 0.05,
        useDocker: true,
        onFinish: () => {
            // do something after the tests have complete
        }
    }
).launch();
```
## Options
**Note:** Only options with `--` can be run with the proceeding flag in the CLI interface.

- `testDir (--t)`: The directory for Mocha to look for the test files
- `testFilter (--f)`: Allows you to pass in some text to filter the test file name, it's just a substring match, nothing fancy
- `shouldRebaseVisuals (--b)`: A flag to tell the visual regression engine to replace the existing baseline visuals
- `reportDir (--r)`: The directory for the Mocha reporter to dump the report files
- `componentTestUrlFactory`: A function that returns the url for the component test to run
- `componentTestVisualPathFactory`: A function that returns the path for visual tests to run in
- `visualThreshold (--v)`: A value between 0 and 1 to present the threshold at which a visual test may pass or fail
- `onFinish`: A function that can be used to do some extra work after Muppeteer is teared down
- `useDocker (--d)`: The option for telling Muppeteer to run Chrome in Docker to better deal with environmental inconsistencies (default)
- `headless (--h)`: Determines whether Chrome will be launched in a headless mode (without GUI) or with a head  (not applicable with `useDocker`)
- `disableSandbox (--s)`: Used to disable the sandbox checks if not using [SUID sandbox](https://chromium.googlesource.com/chromium/src/+/master/docs/linux_suid_sandbox_development.md) (not applicable with `useDocker`)
- `executablePath (--e)`: The option to set the version of Chrome to use duning the tests. By default, it uses the bundled version (not applicable with `useDocker`)

## Puppeteer Extensions
You can access the [Puppeteer Extensions API](https://github.com/HuddleEng/puppeteer-extensions/blob/master/README.md) with `page.extensions` in your tests.
## Example test case

```javascript
const container = '.todoapp';
const input = 'header input';
const listItem = '.todo-list li';
const firstItem = listItem + ':nth-of-type(1)';
const firstItemToggle = firstItem + ' .toggle';
const firstItemRemoveButton = firstItem + ' button';
const secondItem = listItem + ':nth-of-type(2)';
const todoCount = '.todo-count';

describeComponent({name: 'todomvc', url: 'http://localhost:3000'}, () => {
    describe('Add a todo item', async () => {
        it('typing text and hitting enter key adds new item', async () => {
            await page.waitForSelector(input);
            await page.type(input, 'My first item');
            await page.keyboard.press('Enter');
            await page.waitForSelector(firstItem);
            assert.equal(await page.extensions.getText(firstItem), 'My first item');
            await assert.visual(container);
        });
        it('clicking checkbox marks item as complete', async () => {
            await page.waitForSelector(firstItemToggle);
            await page.click(firstItemToggle);

            // something to break the tests
            // await page.addStyleTag({ content: '.header { padding-top: 50px; }'});

            await page.extensions.waitForNthSelectorAttributeValue(listItem, 1, 'class', 'completed');
            await assert.visual(container);
        });
        it('typing more text and hitting enter adds a second item', async () => {
            await page.type(input, 'My second item');
            await page.keyboard.press('Enter');
            await page.waitForSelector(secondItem);
            assert.equal(await page.extensions.getText(secondItem), 'My second item');
            await assert.visual(container);
        });
        it('hovering over first item shows x button', async () => {
            await page.hover(firstItem);
            await assert.visual(container);
        });
        it('clicking on first item x button removes it from the list', async () => {
            await page.click(firstItemRemoveButton);
            await page.extensions.waitForElementCount(listItem, 1);
            assert.equal(await page.extensions.getText(todoCount), '1 item left');
            await assert.visual(container);
        });
    });
});

```

## Docker and test fixtures
Muppeteer uses Docker by default to run tests. This helps to avoid environmental differences that could affect the 
rendering of content on the page. You can opt out by configuring the `useDocker (--d)` option accordingly.

If you are hosting your test fixtures on a local web server, you'd typically set the URL in the test to
something like http://localhost:3000. When using Docker, the `localhost` will refer to the container,
not the host machine. The simplest solution would be to reference the local IP in the test instead. For example,
http://192.168.0.4:3000.

However, this breaks down when you are running on a device you don't know how to address, e.g. a cloud CI agent.
To solve this problem, you can use the `componentTestUrlFactory` function in launch configuration to generate the URL. 
You can lookup the IP address of the current host and pass that through. This is used to run the example tests in this repo. 
See [run-tests](https://github.com/HuddleEng/Muppeteer/blob/master/example/run-tests.js) for an example.

```javascript
...
componentTestUrlFactory: () => `http://${IP}:${PORT}`
...
```

### Passing test output
![Passing tests](https://i.imgur.com/EOA3rJ6.png "Passing tests")

### Failing test output
![Failing tests tests](https://i.imgur.com/rPY6Bjq.png "Failing tests")

### Understanding visual failures

#### Baseline image ####
This is the visual that is versioned in your app repo. It is the source of truth.

![Baseline image](https://i.imgur.com/8dlSqyT.png "Baseline")

#### Current image ####
This is the screenshot taken during the test. In this example, we can see that some padding has 
pushed the text input field down.

![Current image](https://i.imgur.com/DVV3jvZ.png "Current")

#### Difference image ####
This is an image showing where the differences are. Each difference is layered on top of one another. Here we can see 
that the "What needs to be done?" placeholder has moved down, and so it's sitting on top of where "My first item" 
previously was. 

![Difference image](https://i.imgur.com/C1wpxc5.png "Difference")
